package org.apache.maven.linkcheck;

/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2003 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" and
 *    "Apache Maven" must not be used to endorse or promote products
 *    derived from this software without prior written permission. For
 *    written permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    "Apache Maven", nor may "Apache" appear in their name, without
 *    prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * ====================================================================
 */

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.maven.linkcheck.validation.LinkValidationItem;
import org.apache.maven.linkcheck.validation.LinkValidationResult;
import org.apache.maven.linkcheck.validation.LinkValidatorManager;

import com.meterware.httpunit.GetMethodWebRequest;
import com.meterware.httpunit.WebConversation;
import com.meterware.httpunit.WebLink;
import com.meterware.httpunit.WebRequest;
import com.meterware.httpunit.WebResponse;

/**
 * @author <a href="mailto:bwalding@apache.org">Ben Walding</a>
 * @version $Id: FileToCheck.java,v 1.3 2003/02/03 14:13:54 bwalding Exp $
 *
 */
public class FileToCheck
{
  private File base;
  private File f;
  private String status = STATUS_OK;
  private String message = "";
  private LinkCheck linkcheck;
  private int successful;
  private int unsuccessful;

  public static final String STATUS_UNKNOWN = null;
  public static final String STATUS_JTIDY_FAILURE = "Unable to tidy source";
  public static final String STATUS_OK = "OK";

  public FileToCheck(File base, File f)
  {
    this.base = base;
    this.f = f;
  }

  private List links = new ArrayList();

  public void check(LinkValidatorManager lvm) throws Exception
  {
    successful = 0;
    unsuccessful = 0;
    status = STATUS_OK;
    message = "";

    try
    {
      WebConversation wc = new WebConversation();
      WebRequest req = new GetMethodWebRequest(f.toURL().toString());
      WebResponse resp = wc.getResponse(req);
      WebLink[] wl = resp.getLinks();
      Map uniqueLinks = new HashMap();
      for (int i = 0; i < wl.length; i++)
      { //It puts the current URL in item 0
        WebLink link = wl[i];
        String href = link.getDOMSubtree().getAttributes().getNamedItem("href").getNodeValue();

        uniqueLinks.put(href, href);
      }

      Iterator iter = uniqueLinks.keySet().iterator();
      while (iter.hasNext())
      {
        String href = (String) iter.next();

        //System.out.println("Link Found: " + href);

        LinkCheckResult lcr = new LinkCheckResult();

        LinkValidationItem lvi = new LinkValidationItem(f, href);
        LinkValidationResult result = lvm.validateLink(lvi);
        lcr.setTarget(href);

        switch (result.getStatus())
        {
          case LinkValidationResult.UNKNOWN :
            unsuccessful++;
            lcr.setStatus("UNKNOWN REF");
            break;
          case LinkValidationResult.VALID :
            successful++;
            lcr.setStatus("OK");
            break;
          case LinkValidationResult.INVALID :
            unsuccessful++;
            lcr.setStatus("NOT FOUND");
            break;
        }

        links.add(lcr);
      }
    }
    catch (Exception e)
    {
      message = e.toString();
      System.err.println(e.toString());
      throw (e);

    }
  }

  /**
   * Returns the message.
   * @return String
   */
  public String getMessage()
  {
    return message;
  }

  /**
   * Returns the status.
   * @return int
   */
  public String getStatus()
  {
    return status;
  }

  /**
   * Sets the message.
   * @param message The message to set
   */
  public void setMessage(String message)
  {
    this.message = message;
  }

  /**
   * Sets the status.
   * @param status The status to set
   */
  public void setStatus(String status)
  {
    this.status = status;
  }

  public List getResults()
  {
    return links;
  }

  /**
   * Returns the successful.
   * @return int
   */
  public int getSuccessful()
  {
    return successful;
  }

  /**
   * Returns the unsuccessful.
   * @return int
   */
  public int getUnsuccessful()
  {
    return unsuccessful;
  }

  public String getName()
  {
    String baseName = base.getAbsolutePath();
    String fileName = f.getAbsolutePath();
    if (fileName.startsWith(baseName))
      fileName = fileName.substring(baseName.length() + 1);

    fileName = fileName.replace('\\', '/');
    return fileName;
  }

  public String toXML()
  {
    StringBuffer buf = new StringBuffer();

    buf.append("  <file>\n");
    buf.append("    <name>" + getName() + "</name>\n");
    buf.append("    <successful>" + getSuccessful() + "</successful>\n");
    buf.append("    <unsuccessful>" + getUnsuccessful() + "</unsuccessful>\n");

    Iterator iter = getResults().iterator();
    while (iter.hasNext())
    {
      LinkCheckResult result = (LinkCheckResult) iter.next();
      buf.append(result.toXML());
    }

    buf.append("  </file>\n");

    return buf.toString();
  }

}
